/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule auto-download-history-dialog-controller
 */

(function() {
var AutoDownloadHistoryDialogController;
var prototype;
var superConstructor;
var AutoDownloadHistoryDialogView = dixim.autoDownload.AutoDownloadHistoryDialogView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientAutoDownload = dixim.common.ApiClientAutoDownload;

/**
 * @class AutoDownloadHistoryDialogController
 * @constructor
 * @param {Object} configs
 */
AutoDownloadHistoryDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(AutoDownloadHistoryDialogController, EventTarget);
dixim.autoDownload.AutoDownloadHistoryDialogController = AutoDownloadHistoryDialogController;
prototype = AutoDownloadHistoryDialogController.prototype;
superConstructor = AutoDownloadHistoryDialogController.superConstructor;

var HISTORY_DELETE = 'historyDelete';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientAutoDownload();
    this._model.addEventListener('fetchTaskList', this._successFetchTaskList, this);
    this._model.addEventListener('fetchTaskListError', this._errorFetchTaskList, this);
    this._model.addEventListener('fetchItemList', this._successFetchItemList, this);
    this._model.addEventListener('fetchItemListError', this._errorFetchItemList, this);
    this._model.addEventListener('removeItem', this._successRemoveItem, this);
    this._model.addEventListener('removeItemError', this._errorRemoveItem, this);

    this._view = new AutoDownloadHistoryDialogView(configs);
    this._view.addEventListener(HISTORY_DELETE, this._clickHistoryDelete, this);

    this._taskList;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();

    this._view.startIndicator();
    this._model.fetchTaskList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successFetchTaskList = function(e) {
    var query = {};

    this._taskList = e.args.tasks;

    query.filter = 'finished';
    this._model.fetchItemList(query);
};

prototype._errorFetchTaskList = function() {
    this._view.stopIndicator();
    this._view.error(_('auto_download_history_fetch_task_error'));
};

prototype.getTaskById = function(taskId) {
    var taskListLen = this._taskList.length;

    for (var i = 0; i < taskListLen; i += 1) {
        if (taskId === this._taskList[i].taskId) {
            return this._taskList[i];
        }
    }
    return null;
};

prototype._successFetchItemList = function(e) {
    var items = e.args.items;
    var itemsLen = items.length;

    if (itemsLen === 0) {
        this._view.stopIndicator();
        this._view.noRow();
        return;
    }

    if (itemsLen > 10000) itemsLen = 10000;

    for (var i = 0; i < itemsLen; i += 1) {
        this._view.addRow(this.getTaskById(items[i].taskId), items[i]);
    }
    this._view.stopIndicator();
};

prototype._errorFetchItemList = function(e) {
    this._view.stopIndicator();
    this._view.error(_('auto_download_history_fetch_item_error'));
};

prototype._clickHistoryDelete = function() {
    this._view.startIndicator();
    this._model.removeItem();
};

prototype._successRemoveItem = function() {
    this._view.stopIndicator();
    this._update();
};

prototype._errorRemoveItem = function() {
    this._view.stopIndicator();
    this._view.error(_('auto_download_history_remove_error'));
};

prototype._update = function() {
    this._view.startIndicator();
    this._view.cleanList();
    this._model.fetchTaskList();
};

}());
