/**
 * @module dixim 
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule modal-dialog-view
 */

(function() {
var ModalDialogView;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Template = dixim.ctt.util.Template;
var Configs = dixim.ctt.Configs;
var UiUtils = dixim.common.UiUtils;
var Utils = dixim.ctt.util.Utils;
var $ = dixim.ctt.jQuery;

/**
 * @class ModalDialogView
 * @constructor
 * @param {Object} configs
 */
ModalDialogView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ModalDialogView, EventTarget);
dixim.common.ModalDialogView = ModalDialogView;
prototype = ModalDialogView.prototype;
superConstructor = ModalDialogView.superConstructor;

var RENDER = 'render';
var CLOSE = 'close';

var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';

    this._bodyContent = configs.bodyContent || '';
    this._body = configs.body;
    this._parentId = configs.parentId || '';
    this._id = [configs.id, 'modal-dialog'].join('');
    this._title = configs.title || '';
    this._contentHeight = configs.contentHeight || 'auto';
    this._contentWidth = configs.contentWidth || 'auto';
    this._buttons = configs.buttons || [];
    this._lists = configs.lists || [];
    this._doubleDialog = configs.doubleDialog || false;
    this._closeButton = configs.closeButton;
    this._returnButton = configs.returnButton;
    this._uiUtils = UiUtils;
    this._indicating = false;

    if (typeof this._body !== 'undefined') {
        if (this._id !== '') {
            $(this._body).prop('id', this._id);
        }
        this._id = $(this._body).prop('id');
    }
    this._createModalDialog();
};

prototype._createModalDialog = function() {
    var lists_length = this._lists.length;
    var list;
    var buttonsLen = this._buttons.length;
    var button;
    var tagDialogSelector = ['#modal-dialog-', this._id].join('');
    var tagDialogBodyContentSelector = ['#body-content-', this._id].join('');
    var tagDialogListsSelector = ['#modal-dialog-lists-', this._id].join('');
    var tagDialogButtonsSelector = ['#modal-dialog-buttons-', this._id].join('');

    if ($('#modal-dialog-background').length < 1) {
        $('body > div[data-role="page"]:first-child').before([
            '<div id="modal-dialog-background" class="modal-dialog-background">', '</div>'
        ].join(''));
    }
    $(tagDialogSelector).remove();
    $('body').append([
        '<div data-role="dialog" id="modal-dialog-', this._id, '" class="modal-dialog" data-overlay-theme="b" data-dismissible="false" style="min-width: 200px; min-height: 200px;">',
        '</div>',
    ].join(''));

    if (this._title !== '') {
        $(tagDialogSelector).append([
            '<div id="modal-dialog-header" class="dialog_header">',
            '<span id="modal-dialog-header-text" class="dialog_header_text">', this._title, '</span>',
            '</div>',
        ].join(''));

        if (typeof this._closeButton !== 'undefined') {
            $([tagDialogSelector, ' #modal-dialog-header'].join('')).append([
                '<a id="modal-dialog-close-btn" class="dlg_hd_right">',
                '<img class="rspd" src="', Configs.get('imagePath'), '/ico_hd_cancel.png" width="100" height="100" alt="#" />',
                '</a>',
            ].join(''));
            $([tagDialogSelector, ' #modal-dialog-close-btn'].join('')).on('click.closeModal', this._closeButton);
        }

        if (typeof this._returnButton !== 'undefined') {
            $([tagDialogSelector, ' #modal-dialog-header'].join('')).append([
                '<a id="modal-dialog-return-btn-', this._id,'" class="dlg_hd_left dg_hd_prev">', '</a>',
            ].join(''));
            $([tagDialogSelector, ' #modal-dialog-return-btn-', this._id].join('')).on('click', this._returnButton);
        }
    }

    if(typeof this._body !== 'undefined') {
        $(tagDialogSelector).append(this._body);
        $([tagDialogSelector, ' #', this._id].join('')).show();
    }

    if (this._bodyContent !== '') {
        $(tagDialogSelector).append([
            '<div id="body-content-', this._id, '" class="modal-dialog-body-content">', '</div>',
        ].join(''));
        $(tagDialogBodyContentSelector)
        .css('height', this._contentHeight)
        .css('width', this._contentWidth);
        $(tagDialogBodyContentSelector).append(this._bodyContent);
    }

    if (lists_length) {
        $(tagDialogSelector).append(['<div id="modal-dialog-lists-', this._id, '" class="modal-dialog-lists">','</div>'].join(''));
    }
    for (var i = 0; i < lists_length; i += 1) {
        list = this._lists[i];
        $(tagDialogListsSelector).append([
            '<button class="mobile-modal-dialog-lists btn" id="', list.text, '">', list.text, '</button>',
        ].join(''));
        $(['#', list.text].join('')).on('click', list.click);
    }

    if (buttonsLen) {
        $(tagDialogSelector).append([
            '<div id="modal-dialog-indicator-', this._id, '" class="modal-dialog-indicator">', '</div>',
            '<div id="modal-dialog-buttons-', this._id, '" class="modal-dialog-buttons">',
            '</div>',
        ].join(''));
    }
    for (var j = 0; j < buttonsLen; j += 1) {
        button = this._buttons[j];
        $([tagDialogSelector,' ', tagDialogButtonsSelector].join('')).append([
            '<label>',
            '<button class="mobile-modal-dialog-buttons" id="', this._id, '-', button.text, '">',
            button.text,
            '</button>',
            '</label>',
        ].join(''));
        if (typeof button.click !== 'undefined') {
            this._uiUtils.setEventListener([this._id, '-', button.text].join(''), 'click', button.click);
        }
    }

    $(tagDialogSelector).on('pagebeforeshow', dixim.proxy(function(e, ui) {
        if (! this._doubleDialog) {
            $(tagDialogSelector).prevAll('[data-role="page"]').addClass('ui-dialog-background');
        } else {
            $(tagDialogSelector).prevAll('[data-role="dialog"]').addClass('ui-dialog-background-clear');
        }
    }, this));
    $(tagDialogSelector).on('pageshow', function(e, ui) {
        $(this).css('z-index', '9999');
    });
    $(tagDialogSelector).on('pagehide', function(e, ui) {
        $(this).css('z-index', '1');
    });
};

/**
 * @method show
 */
prototype.show = function() {
    var tagDialogSelector = ['#modal-dialog-', this._id].join('');
    var winHeight = $(window).height();
    var dialogHeight;

    if (! this._doubleDialog) {
        $('article').css('overflow', 'hidden');
    }
    if (Utils.isJqueryMobile()) {
        $(':mobile-pagecontainer').pagecontainer('change', tagDialogSelector, {transition: 'none', changeHash: false});
        $(tagDialogSelector).css('display', 'block');
    } else {
        this._pageChange();
    }
    this._render();
    dialogHeight = $(tagDialogSelector).children('.ui-dialog-contain').height();
    $(tagDialogSelector).css('padding-top', (winHeight - dialogHeight) / 2);
    $(tagDialogSelector).css('padding-bottom', (winHeight - dialogHeight) / 2);
    this._uiUtils.addEffect([tagDialogSelector, ' .ui-btn'].join(''));
    this._uiUtils.addEffect('#modal-dialog-header > a, .dlg_hd > a');
    if ($('body').width() === 375) {
        $('.ui-dialog-contain').css('width', '88%');
    }
};

/**
 * @method hide
 */
prototype.hide = function() {
    if ($(['#modal-dialog-', this._id].join('')).length === 0) return;

    if (this._doubleDialog) {
        $(['#modal-dialog-', this._id].join('')).prev().removeClass('ui-dialog-background-clear');
        $(['#modal-dialog-', this._id].join('')).prev().css('z-index', '9999');
        $(['#modal-dialog-', this._id].join('')).prev().addClass('ui-page-active');
        $(['#modal-dialog-', this._id].join('')).hide().remove();
    } else {
        $(['#modal-dialog-', this._id].join('')).prevAll('[data-role="page"]').removeClass('ui-dialog-background');
        $(['#modal-dialog-', this._id].join('')).prevAll('[data-role="page"]').addClass('ui-page-active');
        $('[data-role="dialog"]').hide().remove();
        $('#modal-dialog-background').remove();
        $('article').css('overflow', 'auto');
    }
    this.dispatchEvent(new Event(CLOSE));
};

prototype._render = function() {
    this.dispatchEvent(new Event(RENDER));
};

prototype._pageChange = function() {
    var tagDialogSelector = ['#modal-dialog-', this._id].join('');

    if (! this._doubleDialog) {
        $(tagDialogSelector).prevAll('[data-role="page"]').addClass('ui-dialog-background').css('z-index', '1');
    } else {
        $(tagDialogSelector).prevAll('[data-role="dialog"]').addClass('ui-dialog-background-clear').css('z-index', '9999');
    }
    $(tagDialogSelector).show();
    $([tagDialogSelector, '> div'].join('')).wrapAll('<div data-role="dialog" class="ui-dialog-contain ui-overlay-shadow ui-corner-all"></div>');
    $(tagDialogSelector)
        .css('z-index', 9999)
        .css('top', 0)
        .css('left', 0);
    $(tagDialogSelector).addClass('ui-page-theme-a ui-dialog ui-page-active');
    $([tagDialogSelector, ', ', tagDialogSelector, ' > div'].join('')).addClass('remote-dialog');
};

prototype.resize = function(size) {
    var tagDialogBodyContentSelector = ['#body-content-', this._id].join('');

    if (typeof size.height !== 'undefined') {
        $(tagDialogBodyContentSelector).css('height', size.height);
    }
    if (typeof size.width !== 'undefined') {
        $(tagDialogBodyContentSelector).css('width', size.width);
    }
};

prototype.destroy = function() {
    $(['#modal-dialog-', this._id].join('')).remove();
};

prototype.clear = function() {
    if (this._bodyContent !== '') {
        $(['#body-content-', this._id].join('')).html(this._bodyContent);
    } else if (this._body !== ''){
        $(['#body-content-', this._id].join('')).html(this._body);
    } else {
        $(['#body-content-', this._id].join('')).html('');
    }
};

prototype.disableButtons = function() {
    $(['#modal-dialog-', this._id, ' .modal-dialog-buttons button'].join('')).prop('disabled', true);
};

prototype.enableButtons = function() {
    $(['#modal-dialog-', this._id, ' .modal-dialog-buttons button'].join('')).prop('disabled', false);
};

prototype.disableButton = function(text) {
    $(['#modal-dialog-', this._id, ' .modal-dialog-buttons button#', this._id, '-', text].join('')).prop('disabled', true);
};

prototype.enableButton = function(text) {
    $(['#modal-dialog-', this._id, ' .modal-dialog-buttons button#', this._id, '-', text].join('')).prop('disabled', false);
};

prototype.setButtons = function(buttons) {
    var buttonsLen = buttons.length;
    var tagDialogSelector = ['#modal-dialog-', this._id].join('');
    var tagDialogButtonsSelector = ['#modal-dialog-buttons-', this._id].join('');

    $([tagDialogSelector, ' ',tagDialogButtonsSelector].join('')).empty();
    for (var j = 0; j < buttonsLen; j += 1) {
        button = buttons[j];
        $([tagDialogSelector,' ', tagDialogButtonsSelector].join('')).append([
            '<label>',
            '<button class="mobile-modal-dialog-buttons" id="', this._id, '-', button.text, '">',
            button.text,
            '</button>',
            '</label>',
        ].join(''));
        $(['#', this._id, '-', button.text].join(''))
            .addClass('ui-btn')
            .addClass('ui-shadow')
            .addClass('ui-corner-all');
        if (typeof button.click !== 'undefined') {
            this._uiUtils.setEventListener([this._id, '-', button.text].join(''), 'click', button.click);
        }
        this._uiUtils.addEffect(['div[role="dialog"] #',this._id, '-', button.text].join(''));
    }
};

prototype.setReturnButton = function(event) {
    var tagDialogSelector = ['#modal-dialog-', this._id].join('');

    $([tagDialogSelector, ' #modal-dialog-header #modal-dialog-return-btn-', this._id].join('')).remove();
    if (typeof event !== 'undefined') {
        $([tagDialogSelector, ' #modal-dialog-header'].join('')).append([
            '<a id="modal-dialog-return-btn-', this._id, '" class="dlg_hd_left dg_hd_prev select-pointer">', '</a>',
        ].join(''));
        $(['#modal-dialog-return-btn-', this._id].join('')).on('click.returnModal', {event: event}, dixim.proxy(this._clickReturnButton, this));        
    }
};

prototype._clickReturnButton = function(e) {
    if (!this._indicating) this.dispatchEvent(new Event(e.data.event));
};

prototype.startIndicator = function() {
    this._indicating = true;
    $(['#modal-dialog-indicator-', this._id].join('')).append(TEMPLATE_LOADING);
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).off('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', true);
};

prototype.startProgressIndicator = function() {
    this._indicating = true;
    $(['#modal-dialog-', this._id, ' div.dlg_main_msg:first-child'].join('')).append(TEMPLATE_LOADING);
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).off('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', true);
};

prototype.startSpecifiedIndicator = function(selector) {
    this._indicating = true;
    $(['#modal-dialog-', this._id].join('')).find(selector).append(TEMPLATE_LOADING);
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).off('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', true);
};

prototype.stopIndicator = function() {
    this._indicating = false;
    $(['#modal-dialog-indicator-', this._id, ' .ctt-loading'].join('')).remove();
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).on('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', false);
};

prototype.stopSpecifiedIndicator = function(selector) {
    this._indicating = false;
    $(['#modal-dialog-', this._id].join('')).find(selector).find('.ctt-loading').remove();
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).on('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', false);
};

prototype.stopProgressIndicator = function() {
    this._indicating = false;
    $(['#modal-dialog-', this._id, ' div.dlg_main_msg:first-child .ctt-loading'].join('')).remove();
    $(['#modal-dialog-', this._id, ' #modal-dialog-close-btn'].join('')).on('.closeModal');
    $(['#modal-dialog-', this._id, ' button'].join('')).prop('disabled', false);
};

}());
