/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule progress-bar
 */

(function() {
var ProgressBar;
var prototype;
var superConstructor;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

/**
 * @class ProgressBar
 * @constructor
 * @param {Object} configs
 */
ProgressBar = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ProgressBar, EventTarget);
dixim.common.ProgressBar = ProgressBar;
prototype = ProgressBar.prototype;
superConstructor = ProgressBar.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._descriptionFlag = configs.descriptionFlag || false;
};

prototype.createProgressBar = function(selector) {
    $(selector).append([
        '<div class="progress_wrap">', '<div class="progress_inner">','</div>', '</div>',
    ].join(''));

    if (this._descriptionFlag) {
        $(selector).append([
            '<div id="progress-description" class="list_detail txt_s">',
            '<p>', '</p>',
            '<p>', _('status_progress_pre_remaining_time'), '</p>',
            '</div>',
        ].join(''));
    }
};

prototype.setProgressBar = function(selector, prog) {
    while (prog > 100) {
        prog -= 100;
    }
    $(selector).find('.progress_inner').css('width', [prog, '%'].join(''));
    if (this._descriptionFlag) {
        $(selector).find('#progress-description p:first-child').html(_('status_progress_finished', Math.floor(prog)));
    }
};

prototype.setRemainingTime = function(selector, time) {
    if (this._descriptionFlag) {
        var time_sec = Math.ceil(time);
        var h = time_sec / 3600 | 0;
        var m = time_sec % 3600 / 60 | 0;
        var s = time_sec % 60;

        if (h > 0) {
            $(selector).find('#progress-description p:last-child').html(_('status_progress_remaining_time_hms', h, m, s));
        } else if (m > 0) {
            $(selector).find('#progress-description p:last-child').html(_('status_progress_remaining_time_ms', m, s));
        } else {
            $(selector).find('#progress-description p:last-child').html(_('status_progress_remaining_time', Math.ceil(time)));
        }
    }
};

prototype.errorProgressBar = function(selector) {
    $(selector).find('.progress_inner').css('width', '100%');
    $(selector).find('.progress_inner').css('background-color', '#f05661');
    if (this._descriptionFlag) {
        $(selector).find('#progress-description p:first-child').html(_('status_progress_finished', 0));
        $(selector).find('#progress-description p:last-child').html(_('status_progress_remaining_time', 0));
    }
};

prototype.setDescriptionFlag = function(flag) {
    this._descriptionFlag = flag;
};

}());
