/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-history-dialog-controller
 */

(function() {
var TransferHistoryDialogController;
var prototype;
var superConstructor;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var TransferHistoryDialogView = dixim.contents.TransferHistoryDialogView;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var PinManager = dixim.ctt.util.PinManager;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;

/**
 * @class TransferHistoryDialogController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 */
TransferHistoryDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(TransferHistoryDialogController, EventTarget);
dixim.contents.TransferHistoryDialogController = TransferHistoryDialogController;
prototype = TransferHistoryDialogController.prototype;
superConstructor = TransferHistoryDialogController.superConstructor;

var CLEAR = 'clear';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.pin
 * @param {MobileTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'transfer-history-dialog'].join('');

    this._model = new ApiClientTransfer();
    this._model.addEventListener('fetchTransferLog', this._successFetchTransferLog, this);
    this._model.addEventListener('fetchTransferLogError', this._errorFetchTransferLog, this);
    this._model.addEventListener('clearTransferLog', this._successClearTransferLog, this);
    this._model.addEventListener('clearTransferLogError', this._errorClearTransferLog, this);

    this._view = new TransferHistoryDialogView(configs);
    this._view.addEventListener(CLEAR, this._clickClear, this);

    this._direction = configs.direction;
    this._pinCode = PinManager.get();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this.fetchTransferLog();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype.fetchTransferLog = function() {
    var query = {};

    if (typeof this._pinCode !== 'undefined') {
        query.pin = this._pinCode;
    }

    this._model.fetchTransferLog(query);
};

prototype._successFetchTransferLog = function(e) {
    this._showLogList(e.args.results);
};

prototype._errorFetchTransferLog = function(e) {
    this._view.error(_('contents_transfer_history_fetch_log_error'));
};

prototype._clearTransferLog = function() {
    this._view.startIndicator();
    this._model.clearTransferLog();
};

prototype._successClearTransferLog = function() {
    this.fetchTransferLog();
    this._view.stopIndicator();
};

prototype._errorClearTransferLog = function() {
    this._view.stopIndicator();
    this._errorDialog(_('contents_transfer_history_clear_error'));
};

prototype._showLogList = function(results) {
    var resultsLen;

    this._view.clearHistoryList();

    for (var i = 0; i < results.length; i += 1) {
        if (this._direction === 'upload') {
            if (results[i].direction !== 2 && results[i].direction !== 4) {
                results.splice(i, 1);
                i -= 1;
                continue;
            }
        } else if (this._direction === 'download') {
            if (results[i].direction !== 1 && results[i].direction !== 3) {
                results.splice(i, 1);
                i -= 1;
                continue;
            }
        } else {
            results.splice(i, 1);
            i -= 1;
        }
    }

    if (results.length === 0) {
        this._view.addNoContents();
        this._view.disableClearButton();
        return;
    }

    if (results.length > 10000) {
        resultsLen = 10000;
    } else {
        resultsLen = results.length;
    }

    for(var j = 0; j < resultsLen; j += 1) {
        this._view.addTransferHistoryList(results[j], j);
    }
};

prototype._clickClear = function() {
    this._clearTransferLog();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('contents_setting_menu_transfer_history');
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
