/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-reservation-dialog-controller
 */

(function() {
var TransferReservationDialogController;
var prototype;
var superConstructor;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var TransferReservationDialogView = dixim.contents.TransferReservationDialogView;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var PinManager = dixim.ctt.util.PinManager;
var Enum = dixim.ctt.util.Enum;
var DeletionDialogController = dixim.contents.DeletionDialogController;
var TransferReservationEditDialogController = dixim.contents.TransferReservationEditDialogController;

/**
 * @class TransferReservationDialogController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 */
TransferReservationDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(TransferReservationDialogController, EventTarget);
dixim.contents.TransferReservationDialogController = TransferReservationDialogController;
prototype = TransferReservationDialogController.prototype;
superConstructor = TransferReservationDialogController.superConstructor;

var CHANGE = 'change';
var DELETE = 'delete';
var CLOSE = 'close';
var UPDATE = 'update';
var Mode = Enum.fetch('DELETION');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientTransfer();
    this._model.addEventListener('fetchReservationList', this._successFetchReservationList, this);
    this._model.addEventListener('fetchReservationListError', this._errorFetchReservationList, this);

    this._view = new TransferReservationDialogView(configs);
    this._view.addEventListener(CHANGE, this._clickChange, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);

    this._pinCode = PinManager.get();
    this._results = [];
    this._type = configs.type;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();

    this._fetchReservationList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchReservationList = function() {
    this._model.fetchReservationList({pin: this._pinCode});
};

prototype._successFetchReservationList = function(e) {
    this._createReservationList(e.args.results);
};

prototype._errorFetchReservationList = function() {
    this._view.error(_('contents_transfer_reservation_fetch_error'));
};

prototype._createReservationList = function(results) {
    var resultsLen = results.length;
    var isExistContents = false;

    this._results  = results;
    if (resultsLen > 10000) {
        resultsLen = 10000;
    }

    for (var i = 0; i < resultsLen; i += 1) {
        if (results[i].type === this._type) {
            isExistContents = true;
            this._view.addReservationList(results[i]);
        }
    }

    if (!isExistContents) {
        this._view.addNoContents();
    }
    this._view.setEventButtons();
    this._view.checkReservationOperate();
};

prototype._clickChange = function(res) {
    var id = res.args;
    var info = {};
    var resultsLen = this._results.length;
    var controller;

    for (var i = 0; i < resultsLen; i += 1) {
        if (id === this._results[i].id) {
            info = this._results[i];
            break;
        }
    }
    controller = new TransferReservationEditDialogController({info: info});
    controller.addEventListener(UPDATE, this._updateTransferHistoryList, this);
    controller.start();
};

prototype._clickDelete = function(e) {
    var dataList = e.args;
    var controller;
    var query = {};

    query.contents = dataList;
    query.mode = Mode.LOG;
    query.doubleDialog = true;
    controller = new DeletionDialogController(query);
    controller.addEventListener(UPDATE, this._updateTransferHistoryList, this);
    controller.start();
};

prototype._updateTransferHistoryList = function() {
    this._view.clearTransferHistoryList();
    this._fetchReservationList();
};

}());
