/**
 * @module dixim
 * @submodule download
 */
dixim.namespace('dixim.download');

/**
 * @module download
 * @submodule download-wizard-select-dst-dialog-view
 */

(function() {
var DownloadWizardSelectDstDialogView;
var prototype;
var superConstructor;
var ModalDialogView = dixim.common.ModalDialogView;
var Configs = dixim.ctt.Configs;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

/**
 * @class DownloadWizardSelectDstDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
DownloadWizardSelectDstDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(DownloadWizardSelectDstDialogView, ModalDialogView);
dixim.download.DownloadWizardSelectDstDialogView = DownloadWizardSelectDstDialogView;
superConstructor = DownloadWizardSelectDstDialogView.superConstructor;
prototype = DownloadWizardSelectDstDialogView.prototype;

var NEXT = 'next';
var SELECT_DIRECTORY = 'selectDirectory';

var TEMPLATE_BODY_CONTENT = [
    '<div id="download-description-field" class="description-field dlg_main_msg">',
    '<span>', '${description}', '</span>',
    '</div>',
    '<div id="download-parent" class="modal_hd_btns">', '</div>',
    '<div id="download-list" class="list-field dlg_scroll list_next">',
    '<table>', '</table>',
    '</div>',
    '<div id="download-error">', '</div>',
].join('');

var CONTENTS_IMAGES = {
    nosecureV: [Configs.get('imagePath'), '/ico_list_nosecure_v.png'].join(''),
    nosecureM: [Configs.get('imagePath'), '/ico_list_nosecure_m.png'].join(''),
    nosecureP: [Configs.get('imagePath'), '/ico_list_nosecure_p.png'].join(''),
    folder: [Configs.get('imagePath'), '/ico_list_folder.png'].join('')
}

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'download-wizard-select-dst-dialog-'].join('');

    configs.title = _('download_wizard_title');

    configs.bodyContent = TEMPLATE_BODY_CONTENT
        .replace('${description}', _('download_wizard_select_dst_description'));

    configs.contentHeight = configs.contentHeight || 'auto';
    configs.buttons = [
        {text: _('dialog_close'), click: dixim.proxy(this.hide, this)},
        {text: _('dialog_next'), click: dixim.proxy(this._clickNext, this)},
    ];

    this._srcInfo;

    superConstructor.prototype._initialize.call(this, configs);
};

prototype.addRow = function(data) {
    var icon = this._getContentImage(data);
    var rowSelector;

    $('#download-list table').append([
        '<tr class="download-task-field-row">', '<td class="select-server-checkbox">',
        '<label class="select-pointer">', '<input type="checkbox" />', '<span>', '</span>', '</label>',
        '</td>', '<td class="select-directory">',
        '<img src="', icon, '" width="40px" />',
        '<span class="info-title-text">', data.title, '</span>',
        '</td>', '</tr>',
    ].join(''));

    rowSelector = $('#download-list table tr:last-child');

    if (data['class'].indexOf('item') !== -1) {
        rowSelector.find('label').removeClass('select-pointer');
        rowSelector.find('[type="checkbox"]').prop('disabled', true);
        rowSelector.addClass('noactive');
    } else {
        rowSelector.find('td').not('.select-server-checkbox').on('click', data, dixim.proxy(this._clickDirectory, this));
        rowSelector.find('[type="checkbox"]').on('click', data, dixim.proxy(this._clickCheckbox, this));
    }
};

prototype.noRow = function() {
    this.error(_('download_wizard_select_dst_no_contents'));
};

prototype._getContentImage = function(info) {
    var srcPath;

    if (info['class'].indexOf('item') !== -1) {
        if (/object.item.videoItem.*/.test(info['class'])) {
            srcPath = CONTENTS_IMAGES['nosecureV'];
        } else if (/object.item.audioItem.*/.test(info['class'])) {
            srcPath = CONTENTS_IMAGES['nosecureM'];
        } else if (/object.item.imageItem.*/.test(info['class'])) {
            srcPath = CONTENTS_IMAGES['nosecureP'];
        }
    } else {
        srcPath = CONTENTS_IMAGES['folder'];
    }
    return srcPath;
};

prototype.changeCheckbox = function() {
    var checkboxSelector = $('#download-list table [type="checkbox"]:checked');

    if (checkboxSelector.length === 1) {
        this.enableButton(_('dialog_next'));
    } else {
        this.disableButton(_('dialog_next'));
    }
};

prototype.error = function(text) {
    $('#download-list').addClass('error-text-field txt_none');
    $('#download-list').html(['<span>', text, '</span>'].join(''));
};

prototype.cleanList = function() {
    $('#download-list').removeClass('error-text-field txt_none');
    $('#download-list').html(['<table>', '</table>'].join(''));
};

prototype._clickDirectory = function(e) {
    this.dispatchEvent(new Event(SELECT_DIRECTORY, e.data));
};

prototype._clickCheckbox = function(e) {
    this.changeCheckbox();
    if (e.currentTarget.checked) {
        this._srcInfo = e.data;
    }
};

prototype._clickNext = function() {
    this.dispatchEvent(new Event(NEXT, this._srcInfo));
};

}());
