/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule media-type-dialog-controller
 */

(function() {
var MediaTypeDialogController;
var prototype;
var superConstructor;
var MediaTypeDialogView = dixim.server.MediaTypeDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var Enum = dixim.ctt.util.Enum;

/**
 * @class MediaTypeDialogController
 * @constructor
 * @param {Object} configs
 */
MediaTypeDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(MediaTypeDialogController, EventTarget);
dixim.server.MediaTypeDialogController = MediaTypeDialogController;
prototype = MediaTypeDialogController.prototype;
superConstructor = MediaTypeDialogController.superConstructor;

var EDIT = 'edit';
var UPDATE = 'update';

var Mode = Enum.fetch('DIALOG');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.title = _('server_public_folder_edit_title');
    configs.mode = Mode.EDIT;

    this._model = new ApiClientServer();
    this._model.addEventListener('removeSharedFolder', this._successRemoveSharedFolder, this);
    this._model.addEventListener('removeSharedFolderError', this._errorRemoveSharedFolder, this);
    this._model.addEventListener('setSharedFolder', this._successSetSharedFolder, this);
    this._model.addEventListener('setSharedFolderError', this._errorSetSharedFolder, this);

    this._view = new MediaTypeDialogView(configs);
    this._view.addEventListener(EDIT, this._clickEdit, this);

    this._sharedFolder = configs.sharedFolder;
    this._mediaType;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successRemoveSharedFolder = function() {
    var query = {};

    query.sharedFolder = this._sharedFolder.path;
    query.mediaType = this._mediaType;
    this._model.setSharedFolder(query);
};

prototype._errorRemoveSharedFolder = function(e) {
    this._view.stopIndicator();
    this._view.error(_('server_public_folder_edit_error'));
};

prototype._successSetSharedFolder = function() {
    this._view.stopIndicator();
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._errorSetSharedFolder = function(e) {
    this._view.stopIndicator();
    this._view.error(_('server_public_folder_edit_error'));
};

prototype._clickEdit = function(e) {
    var query = {};

    this._view.startIndicator();
    this._mediaType = e.args.mediaType;

    query.sharedFolder = this._sharedFolder.path;
    this._model.removeSharedFolder(query);
};

}());
