/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule pairing-view
 */

(function() {
var PairingView;
var prototype;
var superConstructor;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;
var escape = dixim.xml.escape;

/**
 * @class PairingView
 * @constructor
 * @param {Object} configs
 */
PairingView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PairingView, EventTarget);
dixim.server.PairingView = PairingView;
prototype = PairingView.prototype;
superConstructor = PairingView.superConstructor;

var SELECT = 'select';
var DELETE = 'delete';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('server_menu_pairing');
};

/**
 * @method show
 */
prototype.show = function() {
    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<div id="pairing_list">',
        '<table class="tbl_list">',
        '<colgroup>',
        '<col span="1" class="checkbox">',
        '<col span="1" class="list_info">',
        '</colgroup>',
        '<tbody>', '</tbody>',
        '</table>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    UiUtils.cleanActionSettingBar();
    UiUtils.cleanActionBar();
    UiUtils.setActionBar('ab_select', dixim.proxy(this._clickSelect, this));
    UiUtils.setActionBar('ab_delete', dixim.proxy(this._clickDelete, this));

    $('.art_tit').text(this._title);
    $('.btn_prev').show().one('click', dixim.proxy(this._clickPrev, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('.rootPage').empty();
};

prototype.addRow = function(data) {
    var date;

    if (typeof data.deviceId === 'undefined' || data.deviceId === '') {
        data.deviceId = 'dummy';
    };
    if (typeof data.friendlyName === 'undefined' || data.friendlyName === '') {
        data.friendlyName = _('server_pairing_no_friendly_name');
    }

    $('#pairing_list table tbody').append([
        '<tr>',
        '<td>',
        '<label class="select-pointer">',
        '<input type="checkbox" id="checkbox_', data.id, '" />',
        '<span>', '</span>',
        '</label>',
        '</td>',
        '<td>',
        '<h4 class="txt_m">', escape(data.friendlyName), '</h4>',
        '<div class="list_detail txt_s">',
        '</div>',
        '</td>',
        '</tr>',
    ].join(''));

    if (data.expireDate !== '') {
        date = Formatter.stringifyDate(Formatter.dcDateToDate(data.expireDate), 'YYYY/MM/DD hh:mm');
        $('#pairing_list tr:last-child .list_detail').append([
            '<p>', _('server_pairing_description_date', date), '</p>',
        ].join(''));
    } else {
        $('#pairing_list tr:last-child .list_detail').append([
            '<p>', _('server_pairing_description_no_date'), '</p>',
        ].join(''));
    }

    if (data.mac !== '') {
        $('#pairing_list tr:last-child .list_detail').append([
            '<p>', _('server_pairing_description_mac_address', data.mac), '</p>',
        ].join(''));
    }
};

prototype.error = function(text) {
    $('#pairing_error').remove();
    $('.contents_inner').append([
        '<div id="pairing_error" class="error-text-field txt_none">',
        '<span>', text, '</span>',
        '</div>',
    ].join(''));
};

prototype.switchSelect = function() {
    var numberOfNotChecked = $('.tbl_list [type="checkbox"]').not(':checked').length;

    if (numberOfNotChecked > 0) {
        $('.tbl_list [type="checkbox"]').prop('checked', true);
    } else {
        $('.tbl_list [type="checkbox"]').prop('checked', false);
    }
};

prototype._clickSelect = function() {
    this.dispatchEvent(new Event(SELECT));
};

prototype._clickDelete = function() {
    var idList = [];
    var splitVal;

    $('.tbl_list [type="checkbox"]:checked').each(function() {
        splitVal = $(this).prop('id').split('_');
        idList.push(splitVal[splitVal.length - 1]);
    });
    this.dispatchEvent(new Event(DELETE, {idList: idList}));
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

prototype.cleanRegisteredDeviceList = function() {
    $('#pairing_list table tbody').empty();
};

}());
