/**
 * @module dixim
 * @submodule vdms
 */
dixim.namespace('dixim.vdms');

/**
 * @module vdms
 * @submodule contents-distribution-setting-view
 */

(function() {
var ContentsDistributionSettingView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;

/**
 * @class ContentsDistributionSettingView
 * @constructor
 * @param {Object} configs
 */
ContentsDistributionSettingView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ContentsDistributionSettingView, EventTarget);
dixim.vdms.ContentsDistributionSettingView = ContentsDistributionSettingView;
prototype = ContentsDistributionSettingView.prototype;
superConstructor = ContentsDistributionSettingView.superConstructor;

var PREV = 'prev';
var APPLY = 'apply';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('vdms_menu_distribution');
};

/**
 * @method show
 */
prototype.show = function() {
    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<div class="dl_tbl">',
        '<dl>',
        '<dt>', _('vdms_distribution_title'), '</dt>',
        '<dd>',
        '<p>', _('vdms_distribution_description'), '</p>',
        '<label id="distribution-enable" class="select-pointer">',
        '<input type="radio" name="distribution-setting" />',
        '<span>', _('vdms_distribution_enable'), '</span>',
        '</label>',
        '<label id="distribution-disable" class="select-pointer">',
        '<input type="radio" name="distribution-setting" />',
        '<span>', _('vdms_distribution_disable'), '</span>',
        '</label>',
        '</dd>',
        '</dl>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    UiUtils.cleanActionSettingBar();
    UiUtils.cleanActionBar();
    UiUtils.setActionBar('ab_apply', dixim.proxy(this._clickApply, this));

    $('.art_tit').text(this._title);
    $('.btn_prev').show().one('click', dixim.proxy(this._clickPrev, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('.rootPage').empty();
};

prototype.setSetting = function(enableDistribute) {
    if (typeof enableDistribute === 'undefined') {
        $('#distribution-enable [type="radio"]').prop('checked', false);
        $('#distribution-disable [type="radio"]').prop('checked', false);
        return;
    }
    if (enableDistribute) {
        $('#distribution-enable [type="radio"]').prop('checked', true);
    } else {
        $('#distribution-disable [type="radio"]').prop('checked', true);
    }
};

prototype._clickApply = function() {
    var checkedId = $('.contents_inner [type="radio"]:checked').closest('label').prop('id');

    this.dispatchEvent(new Event(APPLY, {enableDistribution: (checkedId === 'distribution-enable')}));
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

}());
