/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule auto-download-api-client
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var ContentsTransferApiClientSession = dixim.ctt.api.ContentsTransferApiClientSession;
var Configs = dixim.ctt.Configs;
var $ = dixim.ctt.jQuery;

var AutoDownloadApiClient;
var prototype;

/**
 * Provides interfaces to Automatic Download Move-in API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class AutoDownloadApiClient
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
AutoDownloadApiClient = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(AutoDownloadApiClient, EventTarget);
dixim.ctt.ad.AutoDownloadApiClient = AutoDownloadApiClient;
prototype = AutoDownloadApiClient.prototype;

/**
 * Triggered when the fetchSourceDeviceList was succeeded.
 *
 * @event fetchSourceDeviceList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.sources
 */

/**
 * Triggered when the fetchSourceDeviceList was failed.
 *
 * @event fetchSourceDeviceListError
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the fetchDestinationStorageList was succeeded.
 *
 * @event fetchDestinationStorageList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.storages
 */

/**
 * Triggered when the fetchDestinationStorageList was failed.
 *
 * @event fetchDestinationStorageListError
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the browse was succeeded.
 *
 * @event browse
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.numberReturned
 * @param {Number} event.args.totalMatches
 * @param {Number} event.args.updateId
 * @param {Object[]} event.args.objects
 */

/**
 * Triggered when the browse was failed.
 *
 * @event browseError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 * @param {Object} event.args.data
 * @param {Number} event.args.data.httpStatus
 * @param {Number} event.args.data.soapErrorCode
 */

/**
 * Triggered when the fetchTaskList was succeeded.
 *
 * @event fetchTaskList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.tasks
 */

/**
 * Triggered when the fetchTaskList was failed.
 *
 * @event fetchTaskListError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the addTask was succeeded.
 *
 * @event addTask
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.taskId
 */

/**
 * Triggered when the addTask was failed.
 *
 * @event addTaskError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateTask was succeeded.
 *
 * @event updateTask
 * @param {Event} event
 */

/**
 * Triggered when the updateTask was failed.
 *
 * @event updateTaskError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the deleteTask was succeeded.
 *
 * @event deleteTask
 * @param {Event} event
 */

/**
 * Triggered when the deleteTask was failed.
 *
 * @event deleteTaskError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the fetchDownloadItemLog was succeeded.
 *
 * @event fetchDownloadItemLog
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object[]} event.args.objects
 */

/**
 * Triggered when the fetchDownloadItemLog was failed.
 *
 * @event fetchDownloadItemLogError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the deleteDownloadItemLog was succeeded.
 *
 * @event deleteDownloadItemLog
 * @param {Event} event
 */

/**
 * Triggered when the deleteDownloadItemLog was failed.
 *
 * @event deleteDownloadItemLogError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the fetchMoveSetting was succeeded.
 *
 * @event fetchMoveSetting
 * @param {Event} event
 * @param {Object} event.args
 * @param {Boolean} event.args.allowMove
 */

/**
 * Triggered when the fetchMoveSetting was failed.
 *
 * @event fetchMoveSettingError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateMoveSetting was succeeded.
 *
 * @event updateMoveSetting
 * @param {Event} event
 */

/**
 * Triggered when the updateMoveSetting was failed.
 *
 * @event updateMoveSettingError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the fetchStorageCapacityThreshold was succeeded.
 *
 * @event fetchStorageCapacityThreshold
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.size
 */

/**
 * Triggered when the fetchStorageCapacityThreshold was failed.
 *
 * @event fetchStorageCapacityThresholdError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateStorageCapacityThreshold was succeeded.
 *
 * @event updateStorageCapacityThreshold
 * @param {Event} event
 */

/**
 * Triggered when the updateStorageCapacityThreshold was failed.
 *
 * @event updateStorageCapacityThresholdError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the deleteDownloadFilterList was succeeded.
 *
 * @event deleteDownloadFilterList
 * @param {Event} event
 */

/**
 * Triggered when the fetchDownloadFilterList was failed.
 *
 * @event fetchDownloadFilterListError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the addFilter was succeeded.
 *
 * @event addFilter
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.filterId
 */

/**
 * Triggered when the addFilter was failed.
 *
 * @event addFilterError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the deleteFilter was succeeded.
 *
 * @event deleteFilter
 * @param {Event} event
 */

/**
 * Triggered when the deleteFilter was failed.
 *
 * @event deleteFilterError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateFilterName was succeeded.
 *
 * @event updateFilterName
 * @param {Event} event
 */

/**
 * Triggered when the updateFilterName was failed.
 *
 * @event updateFilterNameError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateCombCond was succeeded.
 *
 * @event updateCombCond
 * @param {Event} event
 */

/**
 * Triggered when the updateCombCond was failed.
 *
 * @event updateCombCondError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the linkFilter was succeeded.
 *
 * @event linkFilter
 * @param {Event} event
 */

/**
 * Triggered when the linkFilter was failed.
 *
 * @event linkFilterError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the unlinkFilter was succeeded.
 *
 * @event unlinkFilter
 * @param {Event} event
 */

/**
 * Triggered when the unlinkFilter was failed.
 *
 * @event unlinkFilterError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the fetchCondList was succeeded.
 *
 * @event fetchCondList
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.filterId
 */

/**
 * Triggered when the fetchCondList was failed.
 *
 * @event fetchCondListError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the addCond was succeeded.
 *
 * @event addCond
 * @param {Event} event
 * @param {Object} event.args
 * @param {Number} event.args.condId
 */

/**
 * Triggered when the addCond was failed.
 *
 * @event addCondError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the deleteCond was succeeded.
 *
 * @event deleteCond
 * @param {Event} event
 */

/**
 * Triggered when the deleteCond was failed.
 *
 * @event deleteCondError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

/**
 * Triggered when the updateCond was succeeded.
 *
 * @event updateCond
 * @param {Event} event
 */

/**
 * Triggered when the updateCond was failed.
 *
 * @event updateCondError
 * @param {Object} event.args
 * @param {Number} event.args.errorCode
 */

prototype._initialize = function() {};

var _makeResponseHandler = function(handler, args) {
  var slice = Array.prototype.slice;
  return $.proxy(function() {
    handler.apply(this, slice.call(arguments, 0).concat(args));
  }, this);
};

var _handleComplete = function(session, event, request) {
  if (request.complete) {
    request.complete(event);
  }
  session.connection = null;
};

var _handleFailureResponse = function(jqXhr, textStatus, errorThrown, session) {
  var event;
  var request = session.request || {};
  var data;

  if (session.isCancelled()) return;

  if (!!jqXhr.responseText) {
    //data = $.parseJSON(jqXhr.responseText);
    data = null;
  } else {
    data = null;
  }

  event = new Event(session.api + 'Error', {session: session, errorCode: jqXhr.status, data: data});
  if (request.error) {
    request.error(event);
  }
  this.dispatchEvent(event);
  _handleComplete.call(this, session, event, request);
};

var _handleSuccessResponse = function(data, textStatus, jqXhr, session) {
  var event;
  var request = session.request || {};

  if (session.isCancelled()) return;

  event = new Event(session.api, data);
  if (request.success) {
    request.success(event);
  }
  this.dispatchEvent(event);
  _handleComplete.call(this, session, event, request);
};

var _makeAjaxParams = function(method, query, handlerArgs) {
  var async_arg = true;
  if (handlerArgs[0].request !== null && typeof handlerArgs[0].request.async !== "undefined") {
    async_arg = handlerArgs[0].request.async;
  }
  return {
    type: method.toUpperCase(),
    data: query,
    dataType: 'json',
    cache: false,
    success: _makeResponseHandler.call(this, _handleSuccessResponse, handlerArgs),
    error: _makeResponseHandler.call(this, _handleFailureResponse, handlerArgs),
    context: this,
    async: async_arg
  };
};

var _doSimpleFetch = function(event, api, request) {
  var session;
  var cfg;

  session = new ContentsTransferApiClientSession(event);
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'GET', null, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + api, cfg);
  return session;
};

/**
 * Fetches Download source device list.
 *
 * @method fetchSourceDeviceList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchSourceDeviceList = function(request) {
  return _doSimpleFetch.call(this, 'fetchSourceDeviceList', '/src/list', request);
};

/**
 * Browses a device.
 *
 * @method browse
 * @param {Object} request
 * @param {String} [request.udn]
 *  The UDN of the device to be browsed. If not specified, local DMS is browsed.
 *  Usable UDNs can be fetched by fetchSourceDeviceList().
 * @param {String} request.objectId
 * @param {String} [request.browseFlag] "children"(default) or "metadata"
 * @param {Number} [request.startingIndex]
 * @param {Number} [request.requestedCount]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.browse = function(request) {
  var query = {};
  var api;

  if (!!request.udn) {
    api = '/src/browse';
    query.udn = request.udn;
  } else {
    api = '/dst/browse';
  }

  query.objectId = request.objectId;
  if (typeof request.id !== "undefined") query.id = request.id;
  query.browseFlag = request.browseFlag === 'metadata' ? 'BrowseMetadata' : 'BrowseDirectChildren';
  query.startingIndex = request.startingIndex || 0;
  query.requestedCount = request.requestedCount || 0;

  session = new ContentsTransferApiClientSession('browse');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + api, cfg);
  return session;
};

/**
 * Fetches destination(local) storage list.
 *
 * @method fetchDestinationStorageList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchDestinationStorageList = function(request) {
  return _doSimpleFetch.call(this, 'fetchDestinationStorageList', '/dst/storage/list', request);
};

/**
 * Fetches automatic download task list.
 *
 * @method fetchTaskList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchTaskList = function(request) {
  return _doSimpleFetch.call(this, 'fetchTaskList', '/task/list', request);
};

/**
 * Adds an automatic download task.
 *
 * @method addTask
 * @param {Object} request
 * @param {String} request.srcUdn
 * @param {String} request.srcContainerId
 * @param {String} request.dstContainerId
 * @param {Boolean} [request.sdFirst] default is false.
 * @param {String} [request.reservationType] "asap"(default) or "timed".
 * @param {Number} [request.timedReservationStartTime]
 *  Start time of timed download in *minutes* from 0 o'clock.
 *  (i.e. 23 o'clock is 60 * 23 = 1380 minutes)
 *  This value is required if the reservationType is "timed".
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.addTask = function(request) {
  var session;
  var cfg;
  var query = {};

  query.srcUdn = request.srcUdn;
  query.srcContainerId = request.srcContainerId;
  query.dstContainerId = request.dstContainerId;
  query.sdFirst = request.sdFirst ? 'true' : 'false';
  query.reservationType = (request.reservationType || 'ASAP').toUpperCase();
  if (query.reservationType === 'TIMED') {
    query.timedReservationStartTime = parseInt(request.timedReservationStartTime, 10);
  }

  session = new ContentsTransferApiClientSession('addTask');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/task/add', cfg);
  return session;
};

/**
 * Updates an automatic download task.
 *
 * @method updateTask
 * @param {Object} request
 * @param {Number} request.taskId
 * @param {Boolean} [request.sdFirst] default is false.
 * @param {String} [request.reservationType] "asap"(default) or "timed".
 * @param {Number} [request.timedReservationStartTime]
 *  Start time of timed download in *minutes* from 0 o'clock.
 *  (i.e. 23 o'clock is 60 * 23 = 1380 minutes)
 *  This value is required if the reservationType is "timed".
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateTask = function(request) {
  var session;
  var cfg;
  var query = {};

  query.taskId = parseInt(request.taskId, 10);
  query.sdFirst = request.sdFirst ? 'true' : 'false';
  query.reservationType = (request.reservationType || 'asap').toLowerCase();
  if (query.reservationType === 'timed') {
    query.timedReservationStartTime = parseInt(request.timedReservationStartTime, 10);
  }

  session = new ContentsTransferApiClientSession('updateTask');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/task/mod', cfg);
  return session;
};

/**
 * Deletes an automatic download task.
 *
 * @method deleteTask
 * @param {Object} request
 * @param {Number} request.taskId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.deleteTask = function(request) {
  var session;
  var cfg;
  var query = {};

  query.taskId = request.taskId;

  session = new ContentsTransferApiClientSession('taskList');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/task/del', cfg);
  return session;
};

/**
 * fetch download item log
 *
 * @method fetchDownloadItemLog
 * @param {Object} request
 * @param {Number(CSV)} request.taskIds
 * @param {String} request.filter
 * @param {Number} request.startingIndex
 * @param {Number} request.requestedCount
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchDownloadItemLog = function(request) {
  var session;
  var cfg;
  var query = {};

  if (typeof request.taskIds !== "undefined") {
    query.taskIds = request.taskIds;
  }
  if (typeof request.filter !== "undefined") {
    query.filter = request.filter;
  }
  if (typeof request.startingIndex !== "undefined") {
    query.startingIndex = request.startingIndex;
  }
  if (typeof request.requestedCount !== "undefined") {
    query.requestedCount = request.requestedCount;
  }

  session = new ContentsTransferApiClientSession('fetchDownloadLog');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/item/list', cfg);
  return session;
};

/**
 * Deletes download logs related to the task specified.
 *
 * @method deleteDownloadItemLog
 * @param {Object} request
 * @param {Number} request.taskId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.deleteDownloadItemLog = function(request) {
  var session;
  var cfg;
  var query = {};

  session = new ContentsTransferApiClientSession('deleteDownloadItemLog');
  if (typeof request !== "undefined") {
    if (typeof request.taskId !== "undefined") {
      query.taskId = request.taskId;
    }
    session.request = request;
  }

  session.query = query;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/item/del', cfg);
  return session;
};

/**
 * Fetches current Move-in settings.
 *
 * @method fetchMoveSetting
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchMoveSetting = function(request) {
  return _doSimpleFetch.call(this, 'fetchMoveSetting', '/settings/move/get', request);
};

/**
 * Updates current Move-in settings.
 *
 * @method updateMoveSetting
 * @param {Object} request
 * @param {Boolean} request.allowMove
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateMoveSetting = function(request) {
  var session;
  var cfg;
  var query = {};

  query.allowMove = request.allowMove ? "true" : "false";

  session = new ContentsTransferApiClientSession('updateMoveSetting');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/settings/move/set', cfg);
  return session;
};

/**
 * Fetches current storage capacity threshold settings.
 *
 * @method fetchStorageCapacityThreshold
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchStorageCapacityThreshold = function(request) {
  return _doSimpleFetch.call(this, 'fetchStorageCapacityThreshold', '/settings/space/get', request);
};

/**
 * Updates current storage capacity threshold settings.
 *
 * @method updateStorageCapacityThreshold
 * @param {Object} request
 * @param {Number} request.size
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateStorageCapacityThreshold = function(request) {
  var session;
  var cfg;
  var query = {};

  query.size = request.size;

  session = new ContentsTransferApiClientSession('updateStorageCapacityThreshold');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/settings/space/set', cfg);
  return session;
};

/**
 * Fetches download filter list.
 *
 * @method fetchDownloadFilterList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchDownloadFilterList = function(request) {
  return _doSimpleFetch.call(this, 'fetchDownloadFilterList', '/filter/list', request);
};

/**
 * Adds an automatic download filter.
 *
 * @method addFilter
 * @param {Object} request
 * @param {String} request.filterName
 * @param {String} request.combCond
 * @param {Number(CSV)} request.taskIds
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.addFilter = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterName = request.filterName;
  query.combCond = request.combCond;
  query.taskIds = request.taskIds || null;

  session = new ContentsTransferApiClientSession('addFilter');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/add', cfg);
  return session;
};

/**
 * Deletes an automatic download filter.
 *
 * @method deleteFilter
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.deleteFilter = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;

  session = new ContentsTransferApiClientSession('filterList');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/del', cfg);
  return session;
};

/**
 * Updates an automatic download filter name.
 *
 * @method updateFilterName
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {String} request.name
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateFilterName = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = parseInt(request.filterId, 10);
  query.name = request.name;

  session = new ContentsTransferApiClientSession('updateFilterName');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/name/mod', cfg);
  return session;
};

/**
 * Updates an automatic download combCond.
 *
 * @method updateCombCond
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {String} request.conbComd
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateCombCond = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = parseInt(request.filterId, 10);
  query.combCond = request.combCond;

  session = new ContentsTransferApiClientSession('updateCombCond');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/comb/mod', cfg);
  return session;
};

/**
 * Link an automatic download filter and task.
 *
 * @method linkFilter
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Number} request.taskId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.linkFilter = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;
  query.taskId = request.taskId;

  session = new ContentsTransferApiClientSession('filterLink');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/link', cfg);
  return session;
};

/**
 * Unlink an automatic download filter and task.
 *
 * @method linkFilter
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Number} request.taskId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.unlinkFilter = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;
  query.taskId = request.taskId;

  session = new ContentsTransferApiClientSession('filterUnlink');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/unlink', cfg);
  return session;
};

/**
 * Fetches download cond list.
 *
 * @method fetchDownloadCondList
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.fetchCondList = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;

  session = new ContentsTransferApiClientSession('fetchCondList');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/cond/list', cfg);
  return session;
};

/**
 * Adds an automatic download cond.
 *
 * @method addCond
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Number} request.name
 * @param {Number} request.op
 * @param {String} request.value
 * @param {String} request.value2
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.addCond = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;
  query.name = request.name;
  query.op = request.op;
  query.value = request.value;
  if (typeof request.value2 !== "undefined") {
    query.value2 = request.value2;
  }

  session = new ContentsTransferApiClientSession('addFilter');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/cond/add', cfg);
  return session;
};

/**
 * Deletes an automatic download cond.
 *
 * @method deleteCond
 * @param {Object} request
 * @param {Number} request.condId
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.deleteCond = function(request) {
  var session;
  var cfg;
  var query = {};

  query.condId = request.condId;

  session = new ContentsTransferApiClientSession('condList');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/cond/del', cfg);
  return session;
};

/**
 * Updates an automatic download cond.
 *
 * @method updateCond
 * @param {Object} request
 * @param {Number} request.filterId
 * @param {Number} request.name
 * @param {Number} request.op
 * @param {String} request.value
 * @param {String} request.value2
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ContentsTransferApiClientSession} session
 */
prototype.updateCond = function(request) {
  var session;
  var cfg;
  var query = {};

  query.filterId = request.filterId;
  query.name = request.name;
  query.op = request.op;
  query.value = request.value;
  if (typeof request.value2 !== "undefined") {
    query.value2 = request.value2;
  }

  session = new ContentsTransferApiClientSession('updateCond');
  session.query = query;
  session.request = request;
  cfg = _makeAjaxParams.call(this, 'POST', query, [session]);
  session.connection = $.ajax(Configs.get('autoDownloadApi') + '/filter/cond/mod', cfg);
  return session;
};


}());
