/**
 * @module dixim
 * @submodule common 
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule top-menu-view
 */

(function() {
var TopMenuView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Svg = dixim.common.Svg;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;

/**
 * @class TopMenuView
 * @constructor
 * @param {Object} configs
 */
TopMenuView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(TopMenuView, EventTarget);
dixim.common.TopMenuView = TopMenuView;
prototype = TopMenuView.prototype;
superConstructor = TopMenuView.superConstructor;

var MENU_SKIP = 'menuSkip';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    var uiUtils = UiUtils;

    configs = configs || {};

    uiUtils.switchTopMenuCode(false);
};

/**
 * @method show
 */
prototype.show = function() {
    var date = new Date();

    if ($('#wrapper').length !== 0) return;

    $('head').append(['<title>', _('app_name'), '</title>'].join(''));
    $('body').append([
        '<div id="wrapper" class="wrapper">',
        '<div class="main">',
        '<div class="container">',
        '<div class="centered">',
        '<div class="logo-dms">', '<div class="svg-wrap">',
        Svg.get('LOGO'),
        '</div>', '</div>',
        '<h2>', _('common_top_welcome'), '</h2>',
        '<div id="main_button" class="main-btn">', '</div>',
        '<a id="menu_skip_btn" class="link-next" href="#">', _('common_top_menu_skip'), '</a>',
        '</div>',
        '</div>',
        '</div>',
        '<footer id="footer" class="footer">',
        '<div class="container">',
        '<p>&copy; ', date.getFullYear(), ' DigiOn,Inc.</p>',
        '</div>',
        '</footer>',
        '</div>',
    ].join(''));

    $('body').append([
        '<div id="overlay" class="overlay">',
        '<div class="container">',
        '<div class="centered">',
        '<section class="ol-item trigger3">',
        '<h2>', _('common_top_setting'), '</h2>',
        '<ul id="overlay_button" class="ol-item-menu">',
        '</ul>',
        '</section>',
        '<a class="ov-btn close-ov">', _('common_top_setting_close'), '</a>',
        '</div>',
        '</div>',
        '</div>',
    ].join(''));

    $('.close-ov').on('click', function () {
        $.when(
            $('#overlay').fadeOut(300).removeClass('show')
        ).done(function() {
            $('body').removeClass('fix'),
            $('.ol-item').show()
        });
        $('#wrapper').off('.noTap');
    });

    $('#menu_skip_btn').on('click', dixim.proxy(this._clickMenuSkip, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
    $('#wrapper,#overlay').remove();
};

prototype.setEffect = function() {
    var winsize = $(window).height();
    var hsize = $('#header').height();
    var fsize = $('#footer').height();
    var msize = (winsize - hsize - fsize);
    var mcsize = $('#main > .container').height();
    var mtsize = (msize/2) - (mcsize/2);

    if (mcsize < msize){
        $('#main').css('margin-top', mtsize + 'px');
    } else {
        $('#main').css('margin-top', 0)
                  .css('padding-bottom', fsize + 'px');
    }
};

prototype.addMainButton = function(data) {
    $('#main_button').append([
        '<a href="#">',
        '<figure>', '<div class="svg-wrap">', data.svg, '</div>', '</figure>',
        '<p>', data.name, '</p>',
        '</a>',
    ].join(''));
    if (typeof data.trigger_name !== 'undefined') {
        $('#main_button a:last-child').addClass(data.trigger_name);
    }
    $('#main_button > a:last-child').on('click', data, dixim.proxy(this._clickMenuButton, this));
};

prototype.addOverlayButton = function(data) {
    $('#overlay_button').append([
        '<li>',
        '<a href="#">',
        '<figure>', '<div class="svg-wrap">', data.svg, '</div>', '</figure>',
        '<p>', data.name, '</p>',
        '</a>',
        '</li>',
    ].join(''));
    $('#overlay_button > li:last-child a').on('click', data, dixim.proxy(this._clickMenuButton, this));
};

prototype._clickMenuButton = function(e) {
    var trigger;
    var selector;
    var data = e.data;

    if (typeof data.event_param !== 'undefined') {
        this.dispatchEvent(new Event(data.event_param));
    }

    if (typeof data.trigger_name === 'undefined') return true;

    selector = $('#overlay').find(['.', data.trigger_name].join(''));

    if($(selector).length){
        $.when(
            $('.ol-item').not(selector).hide(),
            $('#overlay').fadeIn(200)
        ).done(function() {
            $('#overlay').addClass('show'),
            $('body').addClass('fix')
        });
        $('#wrapper').on('click.noTap', function(e) { e.preventDefault();});
    }
};

prototype._clickMenuSkip = function() {
    this.dispatchEvent(new Event(MENU_SKIP));
};

prototype.clean = function() {
    $('#wrapper, #overlay').remove();
};

}());
