/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule operation-menu-dialog-view
 */

(function() {
var OperationMenuDialogView;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ModalDialogView = dixim.common.ModalDialogView;
var Enum = dixim.ctt.util.Enum;

/**
 * @class OperationMenuDialogView
 * @constructor
 * @param {Object} configs
 */
OperationMenuDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(OperationMenuDialogView, ModalDialogView);
dixim.contents.OperationMenuDialogView = OperationMenuDialogView;
prototype = OperationMenuDialogView.prototype;
superConstructor = OperationMenuDialogView.superConstructor;

var DELETE = 'delete';
var RENAME = 'rename';
var MOVE = 'move';
var MKDIR = 'mkdir';

var OperationMenu = Enum.fetch('OPERATION_MENU');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    var isOnFlag = function() {
        for (var i = 0, len = arguments.length; i < len; i += 1) {
            if (!Enum.isOnFlag(configs.operationStatus, OperationMenu[arguments[i]])) return false;
        }
        return true;
    };
    var isOffFlag = function() {
        for (var i = 0, len = arguments.length; i < len; i += 1) {
            if (!Enum.isOffFlag(configs.operationStatus, OperationMenu[arguments[i]])) return false;
        }
        return true;
    };

    configs.id = configs.id || '';
    configs.id = [configs.id, 'operation-menu-dialog'].join('');

    configs.title = _('dialog_title_operation');
    configs.closeButton = dixim.proxy(this.hide, this);
    configs.buttons = [
        {text: _('dialog_close'), click: dixim.proxy(this.hide, this)},
    ];

    configs.lists = [];
    if (isOffFlag('NO_SELECTED', 'SEARCH_PAGE', 'CATEGORY_CONTENTS') ||
        (isOnFlag('CATEGORY_CONTENTS') && isOffFlag('NO_SELECTED')) ||
        (isOnFlag('SEARCH_PAGE') && isOffFlag('NO_SELECTED'))
    ) {
        configs = this._addDelete(configs);
    }
    if (isOffFlag('NO_SELECTED', 'MULTIPLE', 'CATEGORY_CONTENTS') ||
        (isOnFlag('CATEGORY_CONTENTS') && isOffFlag('NO_SELECTED', 'MULTIPLE', 'CONTAINER'))
    ) {
        configs = this._addRename(configs);
    }
    if (isOffFlag('NO_SELECTED', 'SEARCH_PAGE', 'CATEGORY_CONTENTS')) {
        configs = this._addMove(configs);
    }
    if (isOffFlag('SEARCH_PAGE', 'CATEGORY_CONTENTS')) {
        configs = this._addMkdir(configs);
    }

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._addDelete = function(configs) {
    configs.lists.push({text: _('contents_operation_menu_delete'), click: dixim.proxy(this._clickDelete, this)});
    return configs;
};

prototype._addRename = function(configs) {
    configs.lists.push({text: _('contents_operation_menu_rename'), click: dixim.proxy(this._clickRename, this)});
    return configs;
};

prototype._addMove = function(configs) {
    configs.lists.push({text: _('contents_operation_menu_move'), click: dixim.proxy(this._clickMove, this)});
    return configs;
};

prototype._addMkdir = function(configs) {
    configs.lists.push({text: _('contents_operation_menu_mkdir'), click: dixim.proxy(this._clickMkdir, this)});
    return configs;
};

prototype._clickDelete = function() {
    this.dispatchEvent(new Event(DELETE));
};

prototype._clickRename = function() {
    this.dispatchEvent(new Event(RENAME));
};

prototype._clickMove = function() {
    this.dispatchEvent(new Event(MOVE));
};

prototype._clickMkdir = function() {
    this.dispatchEvent(new Event(MKDIR));
};

}());
